/*
** Copyright (C) Red Hat, Inc. 2005-2008
**
** This program is free software; you can redistribute it and/or modify it
** under the terms of the GNU General Public License version 2 as
** published by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; see the file COPYING. If not, write to the
** Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
** MA 02139, USA.
*/

/*
 * Author: Stanko Kupcevic <kupcevic@redhat.com>
 */

#ifndef __CONGA_MODCLUSTERD_CLUSTERMONITOR_H
#define __CONGA_MODCLUSTERD_CLUSTERMONITOR_H

#include "Cluster.h"
#include "counting_auto_ptr.h"
#include "clumond_globals.h"

#include "String.h"

namespace ClusterMonitoring
{


class ClusterMonitor
{
	public:
		ClusterMonitor(const String& socket_path=MONITORING_CLIENT_SOCKET);
		virtual ~ClusterMonitor();

		// Set cache expiration to age seconds.  0 means 
		// always refresh when we call get_cluster()
		unsigned int expire_cache(unsigned int age);

		counting_auto_ptr<Cluster> get_cluster();
		counting_auto_ptr<Cluster> get_cluster(unsigned int age);
		counting_auto_ptr<Cluster> get_cache();

	private:
		String _sock_path;
		struct timespec last_update;
		counting_auto_ptr<Cluster> cached_copy;
		unsigned int cache_lifetime;

		ClusterMonitor(const ClusterMonitor&);
		ClusterMonitor& operator= (const ClusterMonitor&);
};

};

#endif
